#'  auxiliary functions for visualization
#'
#' @seealso \code{\link[Matrix]{image-methods}, \link{lpolygon}}
#' @author Youngser Park
#' @export

plotmemb <- function(X,cl,inorder=FALSE,main="",drawborder=TRUE,lwd=0.5,lwdb=1,lcol=2,xlab="",ylab="",sub="",useRaster=FALSE,...)
{
    suppressMessages(require(lattice))
    suppressMessages(require(Matrix))
    if (inorder) {
        tmp <- sort(table(cl),dec=TRUE)
    } else {
        tmp <- table(cl)
    }
    tmp <- as.numeric(names(tmp))
    ind2 <- unlist(sapply(1:max(tmp), function(x) which(cl==tmp[x])))
    X2 <- as.matrix(X)
    AS <- Matrix(X2[ind2,ind2])
    p <- Matrix::image(AS,lwd=lwd,useRaster=useRaster,main=main,sub=sub,xlab=xlab,ylab=ylab,...)
    print(p)

    if (drawborder) {
        trellis.focus("panel", 1, 1, highlight=FALSE)
        if (length(lcol)==1) lcol <- rep(lcol,max(cl))
        nc <- table(cl)
        nc.x <- nc.y <- c(1,cumsum(nc)+1)
        for (i in 1:max(cl)) {
            x1 <- nc.x[i]; x2 <- nc.x[i+1]
            y1 <- nc.y[i]; y2 <- nc.y[i+1]
            lpolygon(c(x1,x1,x2,x2), c(y1,y2,y2,y1),border=lcol[i],lwd=lwdb)
        }
        trellis.unfocus()
    }
}


#'  auxiliary functions for visualization
#'
#' @seealso \code{\link{image2}}
#' @author Youngser Park
#' @export

my.image2 <- function (x, col = c("white", rev(heat.colors(23))), breaks,
                       log = TRUE, border = NA, box.offset = 0.1, round = 3, cex,
                       text.cex = 1, text.col = "black", label.col = "black",
                       mar = c(1, 3, 3, 1),
                       labels = 2:3, upper.tri = FALSE, diag.lab = NULL,
                       label.offset = 0.1, label.cex = 1, srt = 90)
{
    if (!is.matrix(x)) {
        x <- t(as.matrix(x))
    }
    if (!missing(cex)) {
#        text.cex = cex
        label.cex = cex
    }
    op <- par(mar = mar, xpd = TRUE)
    x <- x[nrow(x):1, , drop = FALSE]
    x1 <- ncol(x)
    y1 <- nrow(x)
    if (length(col) == 1) {
        col <- rep(col, 3)
    }
    if (length(col) == 2) {
        col <- c(col, col[-1])
    }
    if (missing(breaks)) {
        breaks <- length(col) - 1
    }
    else {
        if (length(breaks) != length(col)) {
            warning("Breaks is not the same length as colors.\nSome blocks may be unfilled or some colors may not be used")
        }
    }
    checkmin <- min(x, na.rm = TRUE)
    if (checkmin < 0) {
        x <- x - checkmin
    }
    missingNA <- is.na(x)
    x[x == 0] <- NA
    if (log) {
        z <- cut(log10(x), breaks)
    }
    else {
        z <- cut(x, breaks)
    }
    z2 <- matrix(z, y1, x1)
    x[is.na(x)] <- 0
    x[missingNA] <- NA
    if (checkmin < 0) {
        x <- x + checkmin
    }
    x <- round(x, round)
    offset <- box.offset/2
    plot(seq(1, x1 + 1, len = 2), seq(1, y1 + 1, len = 2), type = "n",
        axes = FALSE, ann = FALSE, xaxs = "i", yaxs = "i")

    for (i in 1:x1) {
        for (j in 1:y1) {
            if (upper.tri & (i+j <= nrow(x))) next
            if (!is.null(diag.lab) & (i+j-1==nrow(x))) next
            if (is.na(z2[j, i])) {
                n1 <- 1
            }
            else {
                n1 <- match(z2[j, i], levels(z)) + 1
            }
            rect(i + offset, j + offset, i + 1 - offset, j +
                1 - offset, border = border, col = col[n1])
            if (!is.na(text.col)) {
                if (text.cex!=0)
                    text(i + 0.5, j + 0.5, x[j, i], cex = text.cex,
                         col = text.col)
            }
        }
    }
    if (2 %in% labels) {
        if (is.null(diag.lab))
            text(1 - label.offset, 1:y1 + 0.5, rownames(x), pos = 2,
                 offset = 0, cex = label.cex, col=label.col)
        else
            text(x1:1 + 1 - label.offset, 1:y1 + 0.5, rownames(x), pos = 2,
                 offset = 0, cex = label.cex, col=label.col)
    }
    if (4 %in% labels)
        text(x1 + 1 + label.offset, 1:y1 + 0.5, rownames(x),
            pos = 4, offset = 0, cex = label.cex, col=label.col)
    if (1 %in% labels)
        text(1:x1 + 0.5, 0.9 - label.offset, colnames(x), pos = 2,
            offset = 0, srt = (0 + srt), cex = label.cex, col=label.col)
    if (3 %in% labels)
        text(1:x1 + 0.8, y1 + 1.1 + label.offset, colnames(x), col=label.col,
            pos = 2, offset = 0, srt = (360 - srt), cex = label.cex)
    par(op)
}

