#'  auxiliary functions to compute the kernel test statistic.
#'
#' @details
#' Given a list of adjacency spectral embedding \code{X.list[[1]]}, \code{X.list[[2]]}, ...
#' compute the matrix of kernel statistics using \code{\link{kernel.stat}}
#' This matrix \code{S} allows us to cluster the subgraphs into motif using e.g., hierarchical agglomerative clustering
#'
#' @param X.list a list of adjacency spectral embedding
#' @param sigma
#'
#' @return S
#' @author Minh Tang
#' @export

computeS <- function(X.list, sigma){
    K <- length(X.list)
    S <- matrix(0, K, K)
    for(i in 1:(K-1)){
        for(j in (i+1):K){
            Tij <- find.transform(X.list[[i]], X.list[[j]])
            S[i,j] <- kernel.stat(X.list[[i]] %*% Tij, X.list[[j]], sigma)
            S[j,i] <- S[i,j]
        }
    }
    return(S)
}

