#'  auxiliary functions to compute the kernel test statistic.
#'
#' @details
#' Compute the kernel MMD-based statistic of Gretton et al.
#' See Eq.(3) in Lyzinski et al.
#' ``Community detection and classification in hierarchical stochastic blockmodels"
#' We use the Gaussian kernel with bandwith sigma.
#' 
#' @param X \eqn{n x d} matrix
#' @param Y \eqn{n x d} matrix
#' @param sigma a scalar
#'
#' @return kernel stat
#' @author Minh Tang
#' @export

kernel.stat <- function(X,Y,sigma=0.2){
    n <- nrow(X)
    m <- nrow(Y)

    tmpXX <- sum(exp(-(as.matrix(dist(X))^2)/(2*sigma^2))) - n
    tmpYY <- sum(exp(-(as.matrix(dist(Y))^2)/(2*sigma^2))) - m
    tmpXY <- sum(exp(-(rect.dist(X,Y))/(2*sigma^2)))

    tmp <- tmpXX/(n*(n-1)) + tmpYY/(m*(m-1)) - 2*tmpXY/(m*n)

    return((m+n)*tmp)
}

