#'  auxiliary functions to compute the kernel test statistic.
#'
#' @details
#' Given \code{X} a \eqn{n x d} matrix and \code{Y} a \eqn{m x d} matrix,
#' this function compute the \eqn{n x m} matrix whose \eqn{ij} element is
#' the Euclidean distance between the \eqn{i}-th row of \code{X}
#' and the \eqn{j}-th row of \code{Y}.
#'
#' @param X \eqn{n x d} matrix
#' @param Y \eqn{n x d} matrix
#'
#' @return D \eqn{n x m} matrix
#' @author Minh Tang
#' @export

rect.dist <- function(X,Y){
    X <- as.matrix(X)
    Y <- as.matrix(Y)
    n <- nrow(X)
    m <- nrow(Y)
    tmp1 <- X%*%t(Y)
    tmp2 <- outer(rep(1, n), rowSums(Y^2))
    tmp3 <- outer(rowSums(X^2), rep(1,m))

    D <- tmp2 - 2*tmp1 + tmp3
    return(D)
}

#' @export
rect.dist.sqrt <- function(X,Y){
    X <- as.matrix(X)
    Y <- as.matrix(Y)
    n <- nrow(X)
    m <- nrow(Y)
    tmp1 <- X%*%t(Y)
    tmp2 <- outer(rep(1, n), rowSums(Y^2))
    tmp3 <- outer(rowSums(X^2), rep(1,m))

    D <- tmp2 - 2*tmp1 + tmp3
    return(sqrt(D))
}
